package de.ozgcloud.bescheid;

import static de.ozgcloud.common.grpc.GrpcUtil.*;

import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.bescheid.common.callcontext.CurrentUserService;
import io.grpc.CallOptions;
import io.grpc.Channel;
import io.grpc.ClientCall;
import io.grpc.ClientInterceptor;
import io.grpc.ForwardingClientCall.SimpleForwardingClientCall;
import io.grpc.Metadata;
import io.grpc.MethodDescriptor;

@Component("bescheidCallContextInterceptor")
public class BescheidCallContextAttachingInterceptor implements ClientInterceptor {

	public static final String BESCHEID_MANAGER_CLIENT_NAME = "OzgCloud_BescheidManager";

	@Autowired
	private CurrentUserService userService;

	// <A> = Request, <B> = Response
	@Override
	public <A, B> ClientCall<A, B> interceptCall(MethodDescriptor<A, B> method, CallOptions callOptions, Channel next) {
		return new CallContextAttachingClientCall<>(next.newCall(method, callOptions));
	}

	final class CallContextAttachingClientCall<A, B> extends SimpleForwardingClientCall<A, B> {

		protected CallContextAttachingClientCall(ClientCall<A, B> delegate) {
			super(delegate);
		}

		@Override
		public void start(Listener<B> responseListener, Metadata headers) {
			headers.merge(buildCallContextMetadata());
			super.start(responseListener, headers);
		}

		private Metadata buildCallContextMetadata() {
			var metadata = new Metadata();

			userService.getUser().getUserId().ifPresent(userId -> metadata.put(HEADER_KEY_USER_ID, userId.getBytes()));
			metadata.put(HEADER_KEY_CLIENT_NAME, BESCHEID_MANAGER_CLIENT_NAME.getBytes());
			metadata.put(HEADER_KEY_REQUEST_ID, generateRequestId().getBytes());

			return metadata;
		}

		// TODO requestId für command handling uebertragen / erstellen
		private String generateRequestId() {
			return UUID.randomUUID().toString();
		}

	}
}
