/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URLConnection;
import java.util.Base64;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Stream;

import jakarta.activation.MimetypesFileTypeMap;

import org.apache.http.entity.ContentType;
import org.springframework.stereotype.Service;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.nachrichten.postfach.AttachmentFile;
import de.ozgcloud.nachrichten.postfach.PersistPostfachNachrichtService;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.vorgang.attached_item.VorgangAttachedItem;
import de.ozgcloud.vorgang.attached_item.VorgangAttachedItemService;
import de.ozgcloud.vorgang.files.FileService;
import de.ozgcloud.vorgang.files.OzgFile;
import de.ozgcloud.vorgang.files.UploadedFilesReference;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

/**
 * @deprecated Temporally replacement for using GRPC Api by MailService
 */
@Deprecated
@Service("persistPostfachNachrichtByCommandService")
@RequiredArgsConstructor
@Log4j2
class PersistPostfachNachrichtByCommandService implements PersistPostfachNachrichtService {

	static final String CLIENT = "OzgCloud_NachrichtenManager";
	static final String ITEM_NAME = "PostfachMail";
	static final String ATTACHMENT_NAME = "PostfachAttachment";

	private final VorgangAttachedItemService attachedItemService;
	private final FileService fileService;

	@Override
	public void persistNachricht(Optional<String> userId, PostfachNachricht nachricht) {
		throw new UnsupportedOperationException("Not implemented.");
	}

	@Override
	public Optional<Map<String, Object>> findById(String postfachNachrichtId) {
		return attachedItemService.findById(postfachNachrichtId).map(VorgangAttachedItem::getItem);
	}

	@Override
	public Stream<Map<String, Object>> findByVorgangAsMap(String vorgangId) {
		return attachedItemService.find(vorgangId, Optional.of(CLIENT), Optional.of(ITEM_NAME)).map(VorgangAttachedItem::getItem);
	}

	@Override
	public void patch(String postfachNachrichtId, Map<String, Object> propertyMap) {
		attachedItemService.forcePatch(postfachNachrichtId, propertyMap);
	}

	@Override
	public Map<String, Object> getById(String postfachNachrichtId) {
		return attachedItemService.getById(postfachNachrichtId).getItem();
	}

	@Override
	public String persistAttachment(String vorgangId, AttachmentFile attachment) {
		var contentType = getTypeByFile(attachment);
		try (var content = attachment.getContent()) {
			var decContent = Base64.getDecoder().decode(content.readAllBytes());

			return fileService.uploadFileStream(
					createUploadedFilesReference(vorgangId),
					createOzgFile(attachment.getName(), contentType, decContent.length),
					Optional.empty(),
					new ByteArrayInputStream(decContent)).toString();
		} catch (IOException e) {
			throw new TechnicalException("Can not read attached file", e);
		}
	}

	UploadedFilesReference createUploadedFilesReference(String vorgangId) {
		return UploadedFilesReference.builder().vorgangId(vorgangId).client(CLIENT).name(ATTACHMENT_NAME).build();
	}

	OzgFile createOzgFile(String fileName, String contentType, long size) {
		return OzgFile.builder().name(fileName).contentType(contentType).size(size).build();
	}

	String getTypeByFile(AttachmentFile attachmentFile) {
		var fileNameMap = URLConnection.getFileNameMap();

		return Optional.ofNullable(fileNameMap.getContentTypeFor(attachmentFile.getName())).orElseGet(() -> getTypeByContent(attachmentFile));
	}

	private String getTypeByContent(AttachmentFile attachmentFile) {
		try (var contentStream = attachmentFile.getContent()) {
			return Optional.ofNullable(URLConnection.guessContentTypeFromStream(contentStream))
					.orElseGet(() -> getByMimeTypes(attachmentFile.getName()));
		} catch (IOException e) {
			LOG.warn("IO-Exception while guessing content type", e);
		}
		return ContentType.APPLICATION_OCTET_STREAM.toString();
	}

	// uses map file: src/main/resources/mime.types
	private String getByMimeTypes(String fileName) {
		var fileTypeMap = new MimetypesFileTypeMap();

		return fileTypeMap.getContentType(fileName);
	}

	@Override
	public Stream<PostfachNachricht> findRueckfragen(String servicekontoType, String postfachId) {
		throw new UnsupportedOperationException("Not implemented.");
	}

	@Override
	public String persistAnswer(String rueckfrageId, PostfachNachricht answer) {
		throw new UnsupportedOperationException("Not implemented.");
	}

	@Override
	public Stream<PostfachNachricht> findAnswers(String serviceKontoType, String postfachId, String rueckfrageId) {
		throw new UnsupportedOperationException("Not implemented.");
	}

}